%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                  Vilanova Pascal Bootcamp, July 2007
%%                  Isabelle Guyon -- isabelle@clopinet.com
%%                  
%%                  Lab2: Introduction to the Spider and CLOP
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: loading and browsing data
%% The object under study is "data"
%% We construct it with the command data(X, Y), 
%% where X is the input matrix and Y the target vector
%% Here are some of the methods to visualize the data:
%%	-	show:		shows to matrix as a heatmap
%%	-	browse:	browse through the individual patterns	
%%	-	scatter:	make scatter plots

% Check your installation
% 1) Chage the directory to this directory (Lab2(
% 2) type at the prompt:

main;		% A dataset should be loaded
D.train	% Should display a data structure

% 3) Display data:
show(D.train);  
browse(D.train); 
scatter(D.train);

% ==> Exercise: Try various versions of show (see help show)

% 4) Load the same dataset "by hand" and look at the size:

data_dir='../Data/ARCENE/'; 
X=load([data_dir 'arcene_train.data']); 
size(X)
Y=load([data_dir 'arcene_train.labels']); 
size(Y)

% Create a data object and examine it:
dat=data(X, Y);
get_y(dat)
[p,n]=get_dim(dat)

% Display data:
show(dat);  
browse(dat); 
scatter(dat);

% 5) Patterns, which are images (Gisette)
% Load data by hand
data_dir='../Data2/GISETTE/';
X=load([data_dir 'gisette_train.data']); 
size(X)
Y=load([data_dir 'gisette_train.labels']); 
size(Y)

% Create a data object and examine it:
dat=data(X, Y);
[p,n]=get_dim(dat)
browse(dat, 2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: training and testing
%% The object under study is "algorithm"
%% and objects derived from it (naive, svm, kridge, etc.)
%% The main methods are:
%%	-	train:	train a model with given training data
%%	-	test:		test a model with given test data

% Train a naive Bayes classifier
[Dat,Model]=train(naive, D.train);
all(Dat.Y==D.train.Y)			% This shows that Y remains unchanged
Dat=test(Model, D.valid);
all(Dat.Y==D.valid.Y)			% This shows that Y remains unchanged

% Important: the result of processing D.valid with the model is in Dat.X
% Dat.Y remains untouched

% Compute the error rate
lst=loss(Dat);				% Compute the error rate
errate=lst.Y				% Same as the next command		
errate=mean(Dat.X.*Dat.Y<0)
errate=mean(Dat.X.*D.valid.Y<0)		% Same again
ber=balanced_errate(Dat.X, D.valid.Y)		% Compute the balanced error rate

% Compute the AUC and plot the ROC curve
roc(Dat);
auc(Dat.X, D.valid.Y)	

% Exercise: Load other datasets of the challenge and try naive Bayes

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Setting the parameters of an algorithm
%% The object under study is "algorithm"
%% and objects derived from it (naive, svm, kridge, etc.)
%% We are going the enter hyperparameters.

default(svc)		% Display the default hyperparameters
% Example 1:
model=svc({'degree=2', 'shrinkage=0.1'});
% Example 2:
model=kridge('shrinkage=10.^-[0:5]');	% A number of ridge values are provided
							% The optimum value by virtual leave-on-out
							% will be selected.

[Dat,Model]=train(model, D.train);
Dat=test(Model, D.valid);
ber=balanced_errate(Dat.X, D.valid.Y)		% Compute the balanced error rate
roc(Dat);
auc(Dat.X, D.valid.Y)	

% Exercise: Play with the hyperparameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Chaining objects
%% The object under study is "chain"
%% We can use it to create compound models with preprocessing, feature selection, 
%% and classification.

model=chain({standardize, s2n('f_max=1100'), normalize, svc})

[Dat,Model]=train(model, D.train);
Dat=test(Model, D.valid);
ber=balanced_errate(Dat.X, D.valid.Y)		% Compute the balanced error rate
roc(Dat);
auc(Dat.X, D.valid.Y)	

% Exercise:
% Try to improve by playing with the svc hp;
% Replace svc by kridge

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Advanced exercises
%% 1) In main.m, change:
%%      DoNotLearn = 1;  
%%      to 
%%      DoNotLearn = 0;  
%%    type at the prompt:
%%      main;
%%    You will get the result of training the baseline model on Arcene.
%% 2) In main.m, change:
%%      dataset= {'arcene'}; 
%%    to 
%%      dataset= {'arcene', 'dexter', 'dorothea', 'gisette', 'madelon'}; 
%%    type at the prompt:
%%      main;
%%    You will get the result of training the baseline model on all
%%    datasets.
%% 3) Look at the code of the baseline models in model_examples.m.
%%    Try some of the other models by setting in main.m modelset to
%%    something else, e.g.:
%%     modelset    = {'optimized'};
%%    Note: some models are really slow on some datasets.

