%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                  Vilanova Pascal Bootcamp, July 2007
%%                  Isabelle Guyon -- isabelle@clopinet.com
%%                  
%%                  Lab2: Introduction to feature selection
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%We are working on Dexter and Madelon.
%If you do not have yet the data and the code, download them first and install them in the sub-directories:
%Data
%Clop

% ============================================================================
% Functions provided:
% ==================

% -- Challenge data processing --
% main.m			Entry point; loads the data, creates a model, train, test, 
% 				save the results. In particular, creates a "Zipped"
% 				subdirectory in which the results are saved as zipped 
% 				archives, ready to email to guyoni@inf.ethz.ch to
% 				get a point.
% model_examples.m Some examples of models. New, contains baselineDexter.
% 
% -- Visualization --
% cmat_display.m	Function to display matrices as a "heatmap"
% inormalize.m	Subroutine of cmat_display.
% scatterplot.m	Visualization of the data matrix as a scatter plots
%                 (suitable for just a few features, e.g. less than 5).
% -- Scoring --
% simple_score.m  Code used to score the results.
% identify.m      Code called by simple_score.
% 
% -- Random probes --
% @probe          Object implementing the probe method. Called with a feature
%                 ranking method as an argument, it performs feature ranking
%                  according to that method and add pvalues and fdr computed
%                  with the probe method. You may use it e.g. with s2n, Ttest, relief, etc.
% 		    * @probe/get_fidx resorts the indices to force the subset returned to 
%                 be ordered. *
%                 * @probe/training:
%                 Note on FDR calculations:
% 		    The FDR can exceed one. Used in the constructor:
% 		    a.fdr_max= default(Inf, [0 Inf]);           % threshold on the fdr 
% rand_probe.m    Function used by @probe to generate random probes from a data object. 
% 		    * Now excludes the original features with zero variance. *
% pval_compute.m  Function used by @probe to compute pvalues from 2 sets of ranking indices
%                 one generated from real data and one form a random data matrix.
% 		    * Now forces the largest pvalue to be one *
% 
% -- Functions to test filters --
% compare_w_probes.m  Compare the analytic caluculations of pvalue and FDR with the results 			        
%                       obtained by the probe method.
% 			  Example:
% 			  [mod_ana, mod_prob]=compare_w_probes('Ttest', D.train, 20000, 10000);
% compare_filters.m   Compare two filter objects.
% 			  Example:
% 			  [filter1, filter2]=compare_filters(Ttest, probe(s2n), D.train)
% compare_n_filters.m Does the same, with an array of filters
% 			  Example:
% 			  compare_n_filters({Ttest, Pearson, Ftest, aucfs}, D.train); 
% 
% -- Filter Objects--
% As part of the regular CLOP:
% s2n: signal-to-noise ratio univariate filter
% relief: relief multivariate filter based on nearest neighbors
% gs: Gram-Schmidt orthogonalization filter (forward selection)
% Note that these filters do not have embedded pvalue computation
%
% -- Ttest criterion --
% @Ttest          Feature ranking object. Ranks features according to the T statistic.
%                 Computes analytically pvalues. Can be used as a template for implementing
%                 other ranking criteria based on statistical tests, e.g. Ftest, Pearson
%                 correlation coefficient, odd ratio, Chi square test, Wilcoxon-Mann-Whitney test.
% 
% -- Fisher criterion --
% fisher_feat.m		A function ranking features according the the Fisher criterion
% run_var.m			A function called by fisher_feat.
% @Ftest			Matlab object implementing the Fisher filter (includes analytically
% 				computed pvalues), but only for the 2-class case.
% 
% -- Pearson correlation coefficient --
% @Pearson			Object implementing the Pearson correlation coefficient.
% 				Uses the Matlab function corrcoef, which also provides pvalues.
% 
% -- AUC criterion --
% auc_feat_select.m		-- Function that computes the AUC and the Mann-Whitney statistic.
% ranksum.m			-- The original Matlab function with some modifications to avoid
% 				    warnings
% @aucfs			-- Matlab object implemented
% 				   We found that, unsing the Matlab function, there is no monotonic
% 				   relashionship between the ranksum statistic (from which the AUC is 
% 				   computed) and the z statistic used to perform the test. We use this
% 				   last value as ranking criterion so that pvalues vary monotonically
% 				   with the ranking criterion.
% 
% ===============================================================================
% Examples and exercises:
% ======================
% 0) Change the directory to this directory (Lab2)
    close all
    clear classes % Some objects are slighly different from Lab1
% 1) Check the path and user options at the top of main.m are the ones you want to use:
%	DoNotLearn = 1;
    global my_dataset
	my_dataset= {'arcene'};
%2) At the prompt type:
	main;
%You may want to try other datasets dexter, madelon, etc.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Explorations with filters
%-- Once the data is loaded, to try the feature selection methods:

% == Simple s2n filter ==
[dat,mod]=train(s2n('f_max=100'), D.train);
% visualize the heat map of the selected features
show(dat);
% compare with 
show(D.train);
% scatter plot of the two fist features
scatter(dat, 1:5);

% ==> Question: what do you observe on the scatter plots?

% == Gram-Schmidt filter ==
[dat,mod]=train(gs('f_max=5'), D.train); 
% Don't use too many features with gs, it is slow and only the first few are relevant
scatter(dat, 1:5);

% == SVC RFE and RFFS (for comparison)==
% Only for those who have installed SVMs and RF
[dat,mod]=train(svcrfe('f_max=5'), D.train); 
scatter(dat, 1:5); 
%[dat,mod]=train(rffs('f_max=5'), D.train); 
%scatter(dat, 1:5);

% ==> Question: what do you observe now on the scatter plots?

% ==> Exercise: chain preprocessing and filters and redo the previous
% explorations. 
% Example: 
% model=chain({shift_n_scale({'factor=1', 'take_log=1'}), s2n}); 
% [dat,mod]=train(model, D.train); etc.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Statistical tests
%-- Once the data is loaded, to try the feature selection methods:

% == Ttest analytic ==
f_max=1000;  % Number of features to select
p_num=10000; % Number of probes (Slow if large, but more precise)

[dat,mod0]=train(Ttest(['f_max=' num2str(f_max)]), D.train);
show(dat); 

clear dat; % save memory
% Save the results
pval0=get_pval(mod0,1);
fdr0=get_fdr(mod0,1);

% == Ttest with probes ==
[dat,mod1]=train(probe(Ttest, {['f_max=' num2str(f_max)], ['p_num=' num2str(p_num)]}), D.train);
pval1=get_pval(mod1,1);
fdr1=get_fdr(mod1,1);

% == Overlay the results: analytic=red; probe=blue == 
figure; plot(pval1); hold on; plot(pval0, 'r');
title('Ttest p-values (analytic=red; probe=blue)'); 
xlabel('rank'); ylabel('pval');
figure; plot(fdr1); hold on; plot(fdr0, 'r');
title('Ttest false discovery rate (analytic=red; probe=blue)'); 
xlabel('rank'); ylabel('FDR'); 

% ==> Exercise: Rerun several time with the same number of probes. Vary the
% number of probes. What do you observe?

% Note: On Dexter and Dorothea, there is a large difference between 
% the analytic and probe results. See Readme2.doc

% == Make comparisons ==
f_max=1000;   % maximum number of features
p_num=1000; % number of probes
[mod_ana, mod_prob]=compare_w_probes('Ttest', D.train, f_max, p_num);
[filter1, filter2]=compare_filters(Ttest, probe(s2n), D.train);
compare_n_filters({Ttest, Pearson, Ftest, aucfs}, D.train); 

%% Note 1:
%% compare_w_probes('Ttest', D.train, Inf, p_num);
%% is the same as:
%% compare_filters(Ttest, probe(Ttest, ['p_num=', num2str(p_num)]), D.train);
%% and also:
%% compare_n_filters({Ttest, probe(Ttest, ['p_num=', num2str(p_num)])}, D.train);
%% You may get slightly different results because of the variance introduced
%% by drawing a fresh set of probes at each run.

%% Note 2: 
%% - the FDR may be a non monotonic function of rank.
%% - for Dexter there are large differences between the pvalues
%%   for the Ttest by both methods. The distribution is very non-normal.
%% See README2
%% Compare:
compare_filters(Ttest, probe(Ttest), D.train);
% Remove constant feature
compare_filters(chain({rmconst('w_min=0'), Ttest}), chain({rmconst('w_min=0'), probe(Ttest)}), D.train);
% Remove small features
compare_filters(chain({Zfilter('w_min=3'), Ttest}), chain({Zfilter('w_min=3'), probe(Ttest)}), D.train);
% Comment

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Multivariate problems
%-- Madelon is very non-linear. Relief finds all the relevant features, not Ttest
my_dataset= {'madelon'};
main;
    
% == Relief with probes (try with Madelon) ==
f_max=50; 
p_num=1000;
[dat,mod]=train(probe(relief, {['f_max=' num2str(f_max)], ['p_num=' num2str(p_num)]}), D.train);
figure; plot(get_fdr(mod,1), 'r'); title('Relief false discovery rate (red) vs. Ttest (blue)'); xlabel('rank'); ylabel('FDR');
% Compare with Ttest and Ttest with probes
[dat,mod]=train(Ttest(['f_max=' num2str(f_max)]), D.train);
hold on; plot(get_fdr(mod,1)); 
[dat,mod]=train(probe(Ttest, {['f_max=' num2str(f_max)], ['p_num=' num2str(p_num)]}), D.train);
hold on; plot(get_fdr(mod,1), 'b--'); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 2: Varying the number of features
%-- Classification performance varies as a function of the number of features
% Pick models from model_examples.m
% Vary the number of features

my_dataset= {'dexter'};
main;

% We are goinf to test the model chaining Ttest and Naive Bayes
% and vary the number of features.

% Simple way:
k_max=floor(log2(size(D.train.X,2)));
cpu0=cputime;
for k=1:k_max
    f_max=2^k;
    model=chain({Ttest(['f_max=' num2str(f_max)]), naive});
    [Resu_train, Model]=train(model, D.train);
    Resu_valid=test(Model, D.valid);
    AUC(k)=auc(Resu_valid.X, Resu_valid.Y);
end
fprintf('Elapsed time = %5.2f seconds\n', cputime-cpu0);

figure; plot([1:k_max], AUC); hold on; plot([1:k_max], AUC, '.');
xlabel('log2(fnum)'); ylabel('AUC');
grid on

% But, this is inefficient because we recompute the feature ranking many
% times. We could do instead:
cpu0=cputime;
[Data_train, Ttest_trained]=train(Ttest, D.train);
Data_valid=test(Ttest_trained, D.valid);
AUC=[];
k_max=floor(log2(size(D.train.X,2)));
for k=1:k_max
    f_max=2^k;
    [Resu_train, Model]=train(naive, data(Data_train.X(:,1:f_max), Data_train.Y));
    Resu_valid=test(Model, data(Data_valid.X(:,1:f_max), Data_valid.Y));
    AUC(k)=auc(Resu_valid.X, Resu_valid.Y);
end
fprintf('Elapsed time = %5.2f seconds\n', cputime-cpu0);

figure; plot([1:k_max], AUC); hold on; plot([1:k_max], AUC, '.');
xlabel('log2(fnum)'); ylabel('AUC');
grid on

% Now, with a small validation set, we may incur a lot of variance if we
% use a single validation set. We may want to do cross-validation

% Join the training and validation set:
pvalid=get_dim(D.valid);
ptrain=get_dim(D.train);
Dtrain=data([D.train.X;D.valid.X], [D.train.Y;D.valid.Y]);
% Randomly draw 10 validation sets
AUC=[];
k_max=floor(log2(size(Dtrain.X,2)));
for j=1:10
    % This is new:
    fprintf('Iteration %d\n', j);
    pidx=randperm(pvalid+ptrain);
    tridx=pidx(1:ptrain);
    teidx=pidx(ptrain+1:ptrain+pvalid);
    [Data_train, Ttest_trained]=train(Ttest,data(Dtrain.X(tridx,:), Dtrain.Y(tridx,1)));
    Data_valid=test(Ttest_trained, data(Dtrain.X(teidx,:), Dtrain.Y(teidx,1)));
    % The rest is the same    
    for k=1:k_max
        f_max=2^k;
        [Resu_train, Model]=train(naive, data(Data_train.X(:,1:f_max), Data_train.Y));
        Resu_valid=test(Model, data(Data_valid.X(:,1:f_max), Data_valid.Y));
        AUC(j, k)=auc(Resu_valid.X, Resu_valid.Y);
    end
end

figure; errorbar([1:k_max], mean(AUC), std(AUC)); hold on; plot([1:k_max], mean(AUC), '.');
xlabel('log2(fnum)'); ylabel('AUC');
grid on

%% Note 1: The error bars using std are pessimistic. We cannot really use the standard
%% error std/sqrt(10) because we are crawing with replacement, but std is
%% pessimistic.

% ==> Exercise: redo this with other datsets
% What do you observe?
% For Arcene for example, what can you say about the error bar?
% Is it surprising that people who used the validation set error to
% select the best model overfitted the data?

% ==> Question: Couldn't we just rank the features with the whole Dtrain
% and then do cross-validation by varying the numner of features instead
% of re-ranking for all splits? What would be wrong?

% ==> Homework: can you write an object, which selects the optimum number
% of features by cross-validation?


