function [mod_ana, mod_prob]=compare_w_probes(filter_name, dat, f_max, p_num)
% [mod_ana, mod_prob]=compare_w_probes(filter_name, dat, f_max, p_num)
% Compare analytic pvalue computations with those obtained with the "probe"
% method.
% filter_name   -- Name of the filter object.
% data          -- Data object.
% f_max         -- Maximum number of features.
% p_num         -- Probe number.

if nargin<3, f_max=20000; end
if nargin<4, p_num=10000; end

% Create filter object
my_filter=feval(filter_name, ['f_max=' num2str(f_max)]);

% == analytic ==
[dt,mod_ana]=train(my_filter, dat);
clear dt; % save memory

% == with probes ==
[dt,mod_prob]=train(probe(my_filter, {['f_max=' num2str(f_max)], ['p_num=' num2str(p_num)]}), dat);
clear dt; % save memory

% == Overlay the results: analytic=red; probe=blue ==
figure; 
plot(get_pval(mod_prob,1)); 
title([filter_name ' p-values (analytic=red; probe=blue)']); 
xlabel('rank'); 
ylabel('pval');
hold on; 
plot(get_pval(mod_ana,1), 'r');

figure; plot(get_fdr(mod_prob,1)); 
title([filter_name ' false discovery rate (analytic=red; probe=blue)']); 
xlabel('rank'); 
ylabel('FDR');
hold on; 
plot(get_fdr(mod_ana,1), 'r');