%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                  Vilanova Pascal Bootcamp, July 2007
%%                  Isabelle Guyon -- isabelle@clopinet.com
%%                  
%%                  Lab4: Introduction to feature selection
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We are working on the Gisette dataset
% For simplicity, we provide both the representation used in the challenge
% with the pixels scrambled and some garbage features added and the
% original pixel representation.

%============================================================================
% Functions and objects provided:
%================================
%
% main.m		Entry point; loads the data, creates a model, train, test, 
%				save the results. In particular, creates a "Zipped"
%				subdirectory in which the results are saved as zipped 
%				archives, ready to email to guyoni@inf.ethz.ch to
%				get a point.
%
% model_examples.m	some examples of models.
%
% @data         Additions to the data object, in particular to browse digits
%
% @convolve:	Preprocessing object performing a 2d convolution of each
%				pattern, assuming it is a square image.
%				Takes a convolution kernel as an argument.
%
% @gauss_ker:	Gaussian kernel.
%
% @exp_ker:		Exponential kernel.
%
% @fourier:		Two dimensional Fourier transform of the images.
%
% @match_filter: Preprocessing object performing a mactch filtering, i.e.
%				the multiplication of the data matrix X of dim (m,n)
%				by the matrix of templates transposed T' of dim (n, f_max)
%				to get the new representation Xnew=X*T' of dim (m, f_max)
%
% @kmeans_bank:	Filter bank object containing a template matrix consisting
%				of f_max cluster centers.
%
% @hadamard_bank: Filter bank object containing the Hadamard transform filter
%				bank, for which f_max=n.
%
% @pca_bank:	Filter bank object containing the first f_max principal
%				components of the training data matrix.

% Examples and exercises:
% ======================
% 0) Change the directory to this directory (Lab2)
% 1) Check the path and user options at the top of main.m are the ones you
% want to use:
%	DoNotLearn = 1;
%   point data directory to DATA2
%	dataset= {'gisette'};
%2) At the prompt type:
	main;
%You may want to try other datasets dexter, madelon, etc.

% Start the browser
    browse(D.train, 2);

% CONVOLUTIONS
% ============
% Create a convolutional kernel object
my_ker=gauss_ker({'dim1=11', 'dim2=11', 'sigma1=2', 'sigma2=2'});
% or
my_ker=exp_ker({'dim1=11', 'dim2=11', 'sigma1=2', 'sigma2=2'});
% show the kernel
show(my_ker);
% Create a convolve object (takes a kernel as an argument) 
my_prepro=convolve(my_ker); 
% Preprocess the data
d=train(my_prepro, D.train);
% Start the browser
browse(d, 2);

% MATCH FILTERS
% =============
% Create a filter bank
my_bank=pca_bank('f_max=25'); % Don't need to set the dimensions right, will "learn" them by training
% or
my_bank=kmeans_bank('f_max=36');
%or
my_bank=hadamard_bank; 
% This last one has some default dimensions, we can look at how it looks like:
show(my_bank);
% Create a match filter object (takes a filter bank as an argument)
my_prepro=match_filter(my_bank);
% Preprocess the data
[d, my_prepro]=train(my_prepro, D.train);
% Show the trained filter bank
show(my_prepro);
% Start the browser (will work only for Hadamard filtering because the other transforms 
%				create only f_max features.)
browse(d, 2);

% OTHER
% =====
% Fourier transform (not implemented with match filters, use fft)
my_prepro=fourier;
% Preprocess the data
d=train(my_prepro, D.train);
% Start the browser
browse(d, 2);

% Perform PCA (slightly different from the match filter version: 
%			no mean subtraction, no scaling by singular values)
pc=pc_extract;
[d, pc]=train(pc, D.train);
browse(data(pc.U'), 2); % Show the principal components

% TIPS TO GET BETTER MODELS
% =========================
% -> Try first the raw pixel representation with the model chain{normalize, svc}, no feature selection
% and the svc parameters we used before 'degree=3', 'shrinkage=3'
% -> Try to smoothe with convolutions chain{convolve(exp_ker), normalize, svc}. For the convolution, an exponential kernel should work well. You can start with the default parameters.
% -> If the svc is slow to converge and/or the training error is far from zero and close 
% to the validation error: you are "underfitting" or constraining your model too much.
% Solution: decrease the shrinkage and/or increase the order of the polynomial.
% -> After some explorations, you can get a better ranking of you model by using cross-validation rather than a single validation set.