% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
%               SAMPLE CODE FOR THE PERFORMANCE PREDICTION CHALLENGE
%             ADAPTED FOR THE FEATURE SELECTION COURSE VILANOVA 2007
%             Isabelle Guyon -- isabelle@clopinet.com -- July 2007
% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% ISABELLE GUYON AND/OR OTHER ORGANIZERS DISCLAIM ANY EXPRESSED OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
% FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY OF NON-INFRIGEMENT OF ANY THIRD PARTY'S 
% INTELLECTUAL PROPERTY RIGHTS. IN NO EVENT SHALL ISABELLE GUYON AND/OR OTHER ORGANIZERS 
% BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF SOFTWARE, DOCUMENTS, 
% MATERIALS, PUBLICATIONS, OR INFORMATION MADE AVAILABLE FOR THE CHALLENGE. 
%
% ---------
% Tested and modified by: 
% Gideon Dror, gideon@mta.ac.il
% Amir Reza Saffari Azar, amir@ymer.org.
% September 2005 - April 2006
% ---------

%clear all

% -o-|-o-|-o-|-o-|-o-|-o-|-o- BEGIN USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% 1) User-defined directories (no slash at the end of the names):
% --------------------------------------------------------------
% The present set up supposes that there are 2 directories of my_root
% Data
% Clop
% containing the data and the CLOP package.
% If they do not yet exist 3 directories are created in my_root
% Results
% Zipped
% Models
% holding the model predictions, zipped files ready to submit to the
% challenge web site, and the CLOP models.

this_dir    = pwd;                  % Change that to the directory of your project
my_root     = this_dir(1:findstr('Lab', this_dir)-1);

data_dir    = [my_root '/Data2'];   % PIXEL GISETTE 
                                    % Path to the five data directories downloaded                                     
                                    % from http://clopinet.com/isabelle/Projects/NIPS2003/
                                    % or http://www.nipsfsc.ecs.soton.ac.uk/datasets/
                                    % (ARCENE, DEXTER, DOROTHEA, GISETTE, and MADELON ).
resu_dir    = [my_root '/Results']; % Where the results will end up.    
zip_dir     = [my_root '/Zipped'];  % Zipped files with results ready to go!
model_dir   = [my_root '/Models'];  % Where the trained models will end up.
                                    % Note: you need not include the models in
                                    % your submissions, unless you want to
                                    % qulify for a bonus entry. In this
                                    % example, the models are save in full
                                    % in the model_dir directory and a lean
                                    % version (with the hyperparameters and
                                    % the smallest trained entities) is
                                    % save in the resu_dir directory.
                                    
ForceOverWrite = 1;                 % Change this value to 0 if you want to be warned when
                                    % a file already exists before saving a result or model.
DoNotLearn = 1;                     % If set to 1, just load CLOP and the data.

code_dir    = [my_root '/Clop'];    % Challenge Learning Objects Package (CLOP).
score_dir   = [my_root '/Score'];

% 2) Choose your data and models
% ------------------------------
dataset= {'gisette'};
modelset    = {'pixelGisette_gauss_conv'};         % You can create your own models by making
                                    % your own chains.
                                    % This should be an array of model example names
                                    % e.g. {'zarbi', 'Prepro+linearSVC'}
                                    % If you leave this array is empty, it will 
                                    % be replaced by a default list of model names.                              
                                    % For a list of model examples, use:
                                    % > model_examples

DoNotLoadTestData=1;              % To save memory, does not load the test data
MergeDataSets=0;                  % If this flag is zero, training is done on the
                                  % training data only. Otherwise training
                                  % and validation data are merged. 
FoldNum=0;                        % If this flag is positive, 
                                  % k-fold cross-validation is performed.
                                  % with k=FoldNum.
% Note: for compatibility reasons, even if training and validation sets
% were merged for training, they are tested separatly at the end so the
% results can be submitted to the challenge web site.
% -o-|-o-|-o-|-o-|-o-|-o-|-o- END USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');

% Set the path and defaults properly; create directories
% ------------------------------------------------------
warning off
addpath(code_dir);
if exist('use_spider_clop.m') == 2, 
    use_spider_clop(code_dir);
elseif exist('code_version.m') == 2, 
    fprintf('Sample code version : %s\n', code_version('sample_code'));
else disp 'ERROR: Wrong code path. Check your directories and path variables.';
    if exist('README.txt') == 2, type README.txt; end
    if exist('Data/README.txt') == 2, type Data/README.txt; end
    if exist('Clop/README.txt') == 2, type Clop/README.txt; end
    return; 
end
warning on
if isempty(modelset), modelset =  model_examples; end
makedir(resu_dir);
makedir(zip_dir);
makedir(model_dir);
makedir(score_dir);

% LOOP OVER DATASETS 
% ==================
for k = 1:length(dataset)
    
	data_name   = dataset{k};
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s      -o-|-o-|-o-|-o-|-o-|-o-\n', upper(data_name));
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n\n');
    
    % Create a data structure and check the data
    %===========================================
    fprintf('-- %s loading data --\n', upper(data_name));
    input_dir   = [data_dir '/' upper(data_name)];
    input_name  = [input_dir '/' data_name];
    
    D   = create_data_struct(input_name, DoNotLoadTestData);
    % Each member of D (D.train, D.valid, and D.test) 
    % is a data object (a structure) with members X and Y (if Y is given). 
    
    % New: compute data statistics
    data_stats(D);
    
    fprintf('-- %s data loaded --\n', upper(data_name));
    % Note: the data are saved as a Matlab structure 
    % so they will load faster the second time around. 

    % Stop here if we just want to set up everything and load data
    if DoNotLearn, break; end
    
    % LOOP OVER MODELS 
    % ================
    for j = 1:length(modelset)
        
        model_name  = modelset{j};
        
        resu_name   = [resu_dir '/' model_name ];
        makedir(resu_name);
        resu_name   = [resu_name '/' data_name];
        
        % Build a model
        %==============
        fprintf('-- %s-%s building  model\n', upper(data_name), upper(model_name));
        my_model    = model_examples(model_name, data_name);
        % Note: "model_examples" calls a model constructor and returns a learning object.
        % Enter at the prompt "> type model_examples" to view the examples.
        % All learning objects have the two methods "train" and "test".
        % To see the data members, type at the prompt "> struct(my_model)".
        fprintf('-- %s-%s model built\n', upper(data_name), upper(model_name));
        % Save the model (untrained) in the result file
        save_model([resu_name , '_model'], my_model, ForceOverWrite, 0);
        % Reload it just to make sure
        my_model    = load_model([resu_name , '_model']);
        fprintf('-- %s-%s model built\n', upper(data_name), upper(model_name));
        
        % Train the model
        %================
        if MergeDataSets
            % Get rid of the validation set (Useful for the challenge
            % because after the validation phase, both training and
            % validation sets are available for training. The goal is then
            % to make predictions on the test set.
            D.train=data([D.train.X;D.valid.X], [D.train.Y;D.valid.Y]);
            rmfield(D, 'valid');
        end
        
        if FoldNum>0
            % Create a CV model
            cv_model=cv(my_model, {['folds=' num2str(FoldNum)], 'store_all=0'});
            fprintf('-- %s-%s performing %d fold cross-validation\n', upper(data_name), upper(model_name), FoldNum);
            % Call the method "train" of the object "cv_model":
            cv_output = train(cv_model, D.train); 
            % Collect the results
            OutX=[]; OutY=[]; ber=[];
            for kk=1:FoldNum, 
                outX=cv_output.child{kk}.X;
                outY=cv_output.child{kk}.Y;
                OutX=[OutX; outX]; 
                OutY=[OutY; outY]; 
                ber(kk)=balanced_errate(outX, outY);
            end
            cvber=balanced_errate(OutX, OutY);
            ebar=std(ber,1)/sqrt(FoldNum);
            fprintf('CV BER=%5.2f+-%5.2f%%\n', 100*cvber, 100*ebar); 
            fprintf('-- %s-%s cross-validation done in %5.2f seconds\n', upper(data_name), upper(model_name), toc);
            fprintf('-- %s-%s training model on all training data\n', upper(data_name), upper(model_name));
        end
        
        % Call the method "train" of the object "my_model":
        tic;
        [train_output my_model] = train(my_model, D.train);  
        ber = balanced_errate(train_output.X, train_output.Y);
        ebar = error_bar(ber, length(find(train_output.Y>0)));
        fprintf('TRAIN BER=%5.2f +-%5.2f%%\n', 100*ber, 100*ebar); 
        fprintf('-- %s-%s model trained in %5.2f seconds\n', upper(data_name), upper(model_name), toc);
        
        % Save the trained model
        %=======================
        fprintf('-- %s-%s saving the model\n', upper(data_name), upper(model_name));
        mresu_name  = [model_dir '/' model_name];
        makedir(mresu_name);
        mresu_name  = [mresu_name '/' data_name '_model'];
        save_model(mresu_name, my_model, ForceOverWrite, 0);
        % Reload it just to make sure
        %my_model    = load_model(mresu_name);
        fprintf('-- %s-%s models saved as %s*\n', upper(data_name), upper(model_name), mresu_name);

        % Test the model
        %===============
        if DoNotLoadTestData | MergeDataSets
            % Now (re-)load everything
            D = create_data_struct(input_name);
        end
        % Note: in order to be able to make valid challenge
        % submissions, here we keep the original data split.
        % Thus the training error will differ from the
        % error rate on all the training data and the validation
        % error will be similar to the training error (since the
        % validation data was used for training.)
        tic;
        fprintf('-- %s-%s testing model\n', upper(data_name), upper(model_name));
        set_name    = fieldnames(D); % 'train', 'valid' and 'test'
        
        for i = 1:length(set_name) % Loop over training, validation, and test set
            % Call the method "test" of the object "my_model" 
            % on D.train, D.valid, or D.test:
            output  = test(my_model, D.(set_name{i})); 
            discriminant{i} = output.X; % The "test" method returns the predicted discriminant values in output.X
            target = output.Y; % If the targets were provided in the dataset, they are copied in output.Y
            if ~isempty(target)
                ber = balanced_errate(discriminant{i},target);
                ebar = error_bar(ber, length(find(target>0)));
                fprintf('%s, BER=%5.2f +-%5.2f%%\n', set_name{i}, 100*ber, 100*ebar); 
            end
        end  
        if MergeDataSets
            fprintf('WARNING: The results correspond to training on both training and validation sets\n');
            fprintf('=======\n');
        end
        fprintf('-- %s-%s model tested in %5.2f seconds\n', upper(data_name), upper(model_name), toc);  
        
        % Save the results
        %=================
        fprintf('-- %s-%s saving the results\n', upper(data_name), upper(model_name));
        for i=1:length(set_name) % Loop over 'train', 'valid', and 'test'
            save_outputs([[resu_name '_' set_name{i}] '.resu'], sign(discriminant{i}), ForceOverWrite);        
            save_outputs([[resu_name '_' set_name{i}] '.conf'], abs(discriminant{i}), ForceOverWrite);        
        end
        fprintf('-- %s-%s results saved as %s*\n', upper(data_name), upper(model_name), resu_name);
        % Save the indices of the features
        idx_feat=get_fidx(my_model);
        save_outputs([resu_name '.feat'], idx_feat, ForceOverWrite);
        fprintf('-- %s-%s results saved as %s*\n', upper(data_name), upper(model_name), resu_name);
        
        fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');
    
    end % Loop over models
end % Loop over datasets

if ~DoNotLearn,

    %% Zip the archives so they are ready to go!
    %%-----------------------------------------
    if ~usejava('jvm'), warning('Java is not loaded, failed to generate ZIP files !!!'); return; end
    for k = 1:length(modelset)
        model_name  = modelset{k};
        zip_name    = zipall(model_name, resu_dir, zip_dir);
        if ~isempty(zip_name)
            fprintf('-- %s zip archive created, see %s --\n', upper(model_name), zip_name);
        end
    end

    % Score the models
    fprintf('-- %s scoring the models --\n');
    simple_score(resu_dir, data_dir, score_dir, 1);
    
end

        
        



    
